function [t,w]=gm_symmAG(N,ab);
%GM_SYMMAG computation of the nodes and weights for a symmetric weight
% function

% This version uses the reduced polynomials and gauss from OPQ 
% Needs the OPQ package (if you don't have it, use gm_symmAGWo)

% see: Fast variants of the Golub and Welsch algorithm for symmetric
% weight functions by G. Meurant and A. Sommariva
% Numer. Algo. 67, v 3 (2014), pp. 491-506

% Input:
% N = cardinality of the rule
%     if N is odd, ab must be computed with N+1 nodes
% ab = 3-term recurrence for the monic orthogonal polynomials
%      same as in Gautschi's OPQ
%      ab(1,2) is the 0th moment
% Caution: this is not the recurrence coefficients generated by
% gm_classicorthopoly. Use gm_r_jacobi instead

% Output:
% t, w = nodes and weights

%
% Authors G. Meurant and A. Sommariva
% June 2012
% Updated July 2015
%

N0 = size(ab,1);
if N0 < N
 error('gm_symmAG: Input array ab is too short')
end

na = norm(ab(:,1));
if na > 0
 error('gm_symmAG: The weight function must be symmetric')
end

% computation of the 3-term recurrence for the reduced weight function

nu = ab(:,2);
nu_odd = nu(1:2:end);
nu_even = nu(2:2:end);
D = [nu(2); nu_odd(2:end) + nu_even(2:end)];
E = [nu(1) / 2; nu_odd(2:end) .* nu_even(1:end-1)];
ab_symm = [D E];

if rem(N,2) == 0
 % N even
 N_symm = N / 2;
 % use gauss from OPQ
 xw = gauss(N_symm,ab_symm);
 % nodes and weights
 x = sqrt(xw(:,1));
 w = xw(:,2);
 % symmetrization
 xw = [-flipud(x) flipud(w); x w];
 
else
 % N odd
 N_symm = (N-1) / 2;
 % Gauss-Radau with 0 as prescribed node
 xw_symm = gm_symmRadau(N_symm,ab_symm,0);
 w0 = 2 * xw_symm(1,2);
 % nodes and weights
 x = sqrt(xw_symm(2:end,1));
 w = xw_symm(2:end,2);
 % symmetrization
 xw = [-flipud(x) flipud(w); 0 w0; x w];
end

t = xw(:,1)';
w = xw(:,2)';


