function [t,w,nloop]=gm_symmMGWo(N,ab);
%GM_SYMMMGWO computation of the nodes and weights for a symmetric weight
% function

% This version uses the reduced matrix and the optimized Golub and
% Welsch function

% see: Fast variants of the Golub and Welsch algorithm for symmetric
% weight functions by G. Meurant and A. Sommariva
% Numer. Algo. 67, v 3 (2014), pp. 491-506

% Input:
% N = cardinality of the rule
%     if N is odd, ab must be computed with N+1 nodes
% ab = 3-term recurrence for the monic orthogonal polynomials
%      same as in Gautschi's OPQ
%      ab(1,2) is the 0th moment
% Caution: this is not the recurrence coefficients generated by
% gm_classicorthopoly. Use gm_r_jacobi instead

% Output:
% t, w = nodes and weights

%
% Authors G. Meurant and A. Sommariva
% June 2012
% Updated July 2015
%

N0 = size(ab,1);
if N0 < N
 error('gm_symmMGWo: Input array ab is too short')
end

na = norm(ab(:,1));
if na > 0
 error('gm_symmMGWo: The weight function must be symmetric')
end

% computation of the reduced matrix in vectors (a,b)

if mod(N,2) == 0
 even = 1;
 Nc = N / 2;
else
 even = 0;
 Nc = fix(N / 2) +1;
end


absd = ab(:,2);
absq = sqrt(absd);

a = zeros(1,Nc);
b = a;

switch even
 
 case 1
  % N even
  a(1) = absd(2);
  b(1) = absq(2) * absq(3);
  
  k = [2:Nc-1];
  a(k) = absd(2*k-1) + absd(2*k);
  b(k) = absq(2*k) .* absq(2*k+1);
  a(Nc) = absd(N) + absd(N-1);
  start = 1;
  
  [t,w,nloop] = gm_GWx(a,b,[],0);
  w = w / sqrt(2);
  
 case 0
  % N odd
  a(1) = absd(2);
  b(1) = absq(2) * absq(3);
  
  k = [2:Nc-1];
  a(k) = absd(2*k-1) + absd(2*k);
  b(k) = absq(2*k) .* absq(2*k+1);
  a(Nc) = absd(N);
  
  [t,w,nloop] = gm_GWx(a,b,[],0);
  % the first node must be zero
  t(1) = 0;
  w = [w(1) w(2:end) / sqrt(2)];
  start = 2;
  
 otherwise
  error('SymmMGWo: This is not possible')
end

xwp = sqrt(t)';

xw(:,1) = [-xwp(end:-1:start,1); xwp];
xw(:,2) = ab(1,2) * ([w(end:-1:start) w]').^2;

t = xw(:,1)';
w = xw(:,2)';




